@@ -848,6 +848,20 @@ static int shutdown_unix_sk(int sk, struct unix_sk_info *ui)
 	return 0;
 }
 
+static int restore_sk_common(int fd, struct unix_sk_info *ui)
+{
+	if (rst_file_params(fd, ui->ue->fown, ui->ue->flags))
+		return -1;
+
+	if (restore_socket_opts(fd, ui->ue->opts))
+		return -1;
+
+	if (shutdown_unix_sk(fd, ui))
+		return -1;
+
+	return 0;
+}
+
 static void revert_unix_sk_cwd(int *prev_cwd_fd, int *root_fd)
 {
 	if (*root_fd >= 0) {
@@ -919,25 +933,17 @@ static int post_open_unix_sk(struct file_desc *d, int fd)
 	int cwd_fd = -1, root_fd = -1;
 
 	ui = container_of(d, struct unix_sk_info, d);
-	if (ui->flags & (USK_PAIR_MASTER | USK_PAIR_SLAVE))
-		return 0;
+	BUG_ON((ui->flags & (USK_PAIR_MASTER | USK_PAIR_SLAVE)) ||
+			(ui->ue->uflags & (USK_CALLBACK | USK_INHERIT)));
 
 	peer = ui->peer;
-
-	if (peer == NULL)
-		return 0;
-
-	if (ui->ue->uflags & USK_CALLBACK)
-		return 0;
+	BUG_ON(peer == NULL);
 
 	/* Skip external sockets */
 	if (!list_empty(&peer->d.fd_info_head))
 		if (peer_is_not_prepared(peer))
 			return 1;
 
-	if (ui->ue->uflags & USK_INHERIT)
-		return 0;
-
 	memset(&addr, 0, sizeof(addr));
 	addr.sun_family = AF_UNIX;
 	memcpy(&addr.sun_path, peer->name, peer->ue->name.len);
@@ -960,16 +966,7 @@ static int post_open_unix_sk(struct file_desc *d, int fd)
 	if (peer->queuer == ui->ue->ino && restore_sk_queue(fd, peer->ue->id))
 		return -1;
 
-	if (rst_file_params(fd, ui->ue->fown, ui->ue->flags))
-		return -1;
-
-	if (restore_socket_opts(fd, ui->ue->opts))
-		return -1;
-
-	if (shutdown_unix_sk(fd, ui))
-		return -1;
-
-	return 0;
+	return restore_sk_common(fd, ui);
 }
 
 static int bind_unix_sk(int sk, struct unix_sk_info *ui)
@@ -1088,7 +1085,6 @@ static int open_unixsk_pair_master(struct unix_sk_info *ui, int *new_fd)
 {
 	int sk[2];
 	struct unix_sk_info *peer = ui->peer;
-	struct fdinfo_list_entry *fle;
 
 	pr_info("Opening pair master (id %#x ino %#x peer %#x)\n",
 			ui->ue->id, ui->ue->ino, ui->ue->peer);
@@ -1106,17 +1102,10 @@ static int open_unixsk_pair_master(struct unix_sk_info *ui, int *new_fd)
 	if (bind_unix_sk(sk[0], ui))
 		return -1;
 
-	if (rst_file_params(sk[0], ui->ue->fown, ui->ue->flags))
+	if (restore_sk_common(sk[0], ui))
 		return -1;
 
-	if (restore_socket_opts(sk[0], ui->ue->opts))
-		return -1;
-
-	if (shutdown_unix_sk(sk[0], ui))
-		return -1;
-
-	fle = file_master(&peer->d);
-	if (send_fd_to_peer(sk[1], fle)) {
+	if (send_desc_to_peer(sk[1], &peer->d)) {
 		pr_err("Can't send pair slave\n");
 		return -1;
 	}
@@ -1129,32 +1118,19 @@ static int open_unixsk_pair_master(struct unix_sk_info *ui, int *new_fd)
 
 static int open_unixsk_pair_slave(struct unix_sk_info *ui, int *new_fd)
 {
-	struct fdinfo_list_entry *fle;
 	int sk, ret;
 
-	fle = file_master(&ui->d);
-
-	pr_info("Opening pair slave (id %#x ino %#x peer %#x) on %d\n",
-			ui->ue->id, ui->ue->ino, ui->ue->peer, fle->fe->fd);
-
-	ret = recv_fd_from_peer(fle);
+	ret = recv_desc_from_peer(&ui->d, &sk);
 	if (ret != 0) {
 		if (ret != 1)
 			pr_err("Can't recv pair slave\n");
 		return ret;
 	}
-	sk = fle->fe->fd;
 
 	if (bind_unix_sk(sk, ui))
 		return -1;
 
-	if (rst_file_params(sk, ui->ue->fown, ui->ue->flags))
-		return -1;
-
-	if (restore_socket_opts(sk, ui->ue->opts))
-		return -1;
-
-	if (shutdown_unix_sk(sk, ui))
+	if (restore_sk_common(sk, ui))
 		return -1;
 
 	*new_fd = sk;
@@ -1291,15 +1267,23 @@ static int open_unixsk_standalone(struct unix_sk_info *ui, int *new_fd)
 		ui->listen = 1;
 		wake_connected_sockets(ui);
 	}
-out:
-	if (rst_file_params(sk, ui->ue->fown, ui->ue->flags))
-		return -1;
 
-	if (restore_socket_opts(sk, ui->ue->opts))
+	if (ui->peer) {
+		/*
+		 * We need to connect() to the peer, but the
+		 * guy might have not bind()-ed himself, so
+		 * let's postpone this.
+		 */
+		*new_fd = sk;
+		return 1;
+	}
+
+out:
+	if (restore_sk_common(sk, ui))
 		return -1;
 
 	*new_fd = sk;
-	return 1;
+	return 0;
 }
 
 static int open_unix_sk(struct file_desc *d, int *new_fd)
